export const colorLegend = (selection, props) => {
  const {                      
    colorScale,                
    circleRadius,
    spacing,                   
    textOffset,
    backgroundRectWidth,
    onClick,
    selectedColorValue
  } = props;      
  
  
  // Setup Circles for continuous color scale
  const begin = colorScale.domain()[0];
  const end = colorScale.domain()[1];
  const n = end - begin;
  let step = n / 6;
  let data = [];
  
  if(n < 6) { step = 1; }
  for(let i = begin; i <= end; i += step) {
    data.push(Math.floor(i));
  }
    
  // Place Color Legend
  selection.attr('transform', 'translate(800,50)')
  
  // Background Rectangle
  const backgroundRect = selection.selectAll('rect').data([null]);             
  backgroundRect.enter().append('rect')
    .merge(backgroundRect)
      .attr('x', -circleRadius * 2)   
      .attr('y', -circleRadius * 2)   
      .attr('rx', circleRadius * 2)   
      .attr('width', backgroundRectWidth)
      .attr('height', (spacing * 7) + (circleRadius * 2)) 
      .attr('fill', 'lightgray')
      .attr('opacity', 0.8);
  

  // Groups For Legend
  const groups = selection.selectAll('.legendFont').data(data);
  const groupsEnter = groups
    .enter().append('g')
      .attr('class', 'legendFont');
  
  groupsEnter
    .merge(groups)
      .attr('transform', (d, i) =>    
        `translate(0, ${i * spacing})`  
      )
      .attr('opacity', d =>
        (!selectedColorValue || d === selectedColorValue)
          ? 1
          : 0.2
      )
  	  .on('click', d => onClick(d === selectedColorValue ? null : d));

  groups.exit().remove();
  
  groupsEnter.append('circle')
    .merge(groups.select('circle')) 
      .attr('r', circleRadius)
      .attr('fill', colorScale)
 
  groupsEnter.append('text')
    .merge(groups.select('text'))   
      .text(d => d)
      .attr('dy', '0.32em')
      .attr('x', textOffset);
}
