import { hierarchy } from 'd3';
export const eventSequences = (selection, props) => {
  const {
    data,
    length,
    margin,
    colorScale,
    selectedPath,
    distributions,
    updateSequence,
    selectedSequence,
    selectedSequenceId,
    selectedSequenceIndex,
  } = props;
  
  
  const root = hierarchy(data);
  const descendants = root.descendants();
  const formattedData = data.map(d => hierarchy(d).descendants());
  
  
  // Selection & Enter for Sequences (Rows)
  const sequences = selection
  	.selectAll('.sequences')
  	.data(formattedData, d => d[0].data.data.id);
  const sequencesEnter = sequences
    .enter().append('g')
      .attr('class', 'sequences')
    .merge(sequences)
      .on('mouseover', (d,i) => updateSequence(d,i))
      .attr('transform', (d,i) => 
            `translate(${margin.right}, ${margin.bottom * i})`);
  sequences.exit().remove();
  
  
  // Selection & Enter for Events (Columns)
  const events = sequencesEnter.selectAll('rect').data((d) => d);
  const eventsEnter = events
    .enter().append('rect')
      .attr('width', length)
      .attr('height', length)
      .attr('stroke', 'black')
      .attr('x', (d,i) => (margin.right + length) * i)
    .merge(events)
      .attr('fill', (d) => 
        (d.data.data.CN === undefined)
        ? 'white'
        : colorScale(distributions[d.data.data.CN])
      );
  
  
  // Draw rectangle around selected sequences
  const selectEnter = selection
    .selectAll(`.sequence-selection`)
    .data([selectedSequenceId], d => d);
  selectEnter
    .enter().append('rect')
      .attr('class', 'sequence-selection')
      .attr('stroke', 'gray')
      .attr('fill', 'yellow')
      .attr('width', ((length + margin.right) 
                      * selectedSequence.length) 
            					+ (margin.right))
      .attr('height', length + (margin.bottom/3))
      .attr('y', (margin.bottom 
                  * selectedSequenceIndex) 
            			- (margin.bottom/6))
      .style('opacity', d => (d === selectedSequenceId) ? 0.3 : 0)
    selectEnter.exit().remove();

  
  // Display info on selected sequence
  const breadBasket = selection
  	.selectAll('.info-background')
    .data([null]);
  breadBasket
    .enter().append('rect')
      .attr('class', 'info-background')
      .attr('fill', '#6e7682')
      .attr('width', 200)
      .attr('rx', 20)
    .merge(breadBasket)
      .attr('height', (21 * selectedSequence.length))
      .attr('x', (margin.right + length) * selectedSequence.length + margin.right * 2 - 5)
      .attr('y', (margin.bottom * selectedSequenceIndex) - (19 * selectedSequence.length/2));
  
  const crumbs = selection
  	.selectAll('.breadcrumbs')
  	.data(selectedSequence, d => String(d.id));
  crumbs
    .enter().append('text')
      .text(d => d.CN)
      .attr('font-size', '20px')
      .attr('class', 'breadcrumbs')
      .attr('fill', d => colorScale(distributions[d.CN]))
      .attr('x', (margin.right + length) * selectedSequence.length + margin.right * 2)
      .attr('y', (d, i) => (margin.bottom * selectedSequenceIndex) 
            								+ (20 * i) + (margin.bottom / 2)
            								- (20 * selectedSequence.length / 2))
  crumbs.exit().remove();
}

