import { select, json, scaleSequential, interpolateRdBu } from 'd3';
import { colorLegend } from './colorLegend';
import { dropdownMenu } from './dropdownMenu';
import { eventSequences } from './eventSequences';


const colorScale = scaleSequential(interpolateRdBu);
const svg = select('svg').attr('transform', `translate(20, 20)`);
const colorLegendG = svg.append('g');
const sequencesG = svg.append('g');

const width = document.body.clientWidth;
const height = document.body.clientHeight;
svg
  .attr('width', width-50)
  .attr('height', height)


// State and Handlers
let data;
let dataChanged = true;

let paths;
let selectedPath;
let distributions = {};
let selectedColorValue;
let selectedSequence = [];
let selectedSequenceId = -1;
let selectedSequenceIndex = -1;
const onColorClicked = (value) => { return; };

const updateSequence = (descendants, index) => {
  selectedSequence = Array();
  descendants.forEach((d, i) => selectedSequence.push(d.data.data));
  
  selectedSequenceId = selectedSequence[0].id;
  selectedSequenceIndex = index;
  render();
}

const onPathClicked = (path) => {
  selectedPath = path;
  selectedSequence = [];
  selectedSequenceId = -1;
  selectedSequenceIndex = -1;
  dataChanged = true;
  render();
}

const applyFilters = () => {
  let filteredData = data;
  if(selectedPath !== 'all') {
    filteredData = data
      .filter(d => d.data.path === selectedPath);
  }
  
  return filteredData;
}

const updateDistributions = (data) => {
  if(dataChanged) {
    distributions = data
      .map(d => d3.hierarchy(d).descendants())
      .reduce((a, currentValue) => {
        currentValue.forEach((d,i) => {
          const val = d.data.data.CN;
          a[val] = (a[val]) ? a[val] + 1 : 1;
        })
        return a;
      }, {});

    delete distributions[undefined];
    dataChanged = false;
  }
}

const render = () => {
  
  let d = [];
  const length = 15;
  const margin = {right: 10, bottom: 23};
  const data = applyFilters();
  updateDistributions(data);
  
  for(let prop in distributions) {
    d.push(distributions[prop]);
  }
  colorScale.domain(d3.extent(d));
  console.log(distributions, d)
  
  sequencesG.call(eventSequences, {
    data,
    length,
    margin,
    colorScale,
    selectedPath,
    distributions,
    updateSequence,
    selectedSequence,
    selectedSequenceId,
    selectedSequenceIndex,
  });
  
  select('#path-menu')
    .call(dropdownMenu, {
      options: paths,
      onOptionClicked: onPathClicked,
      selectedOption: selectedPath
    });
  
  colorLegendG.call(colorLegend, {
    title: 'Distribution Values',
  	spacing: 20,
    colorScale,
    textOffset: 20,
    circleRadius: 10,
    backgroundRectWidth: 80,
    selectedColorValue,
    onClick: onColorClicked,
    width: +svg.attr('width'),
    height: +svg.attr('height'),
  })
  
  
  // Reset height of viz div
  const height = svg.node().getBBox().height + 60;
  const content = document.getElementById('viz');
  // content.setAttribute("style", `height:${height};`);
  svg.attr('height', height);
}



json('data.json').then(loadedData => {
  data = loadedData;
  
  
  // Extract Node CN Distributions
  updateDistributions(data);

  
  // Extract Unique Paths
  paths = data
    .map(d => d.data.path)
    .filter((value, index, self) => self.indexOf(value) === index)
  paths.push('all');

  
  selectedPath = 'all';
  render();
});

